# Copyright (C) 2020 VMware, Inc. All rights reserved.
# -- VMware Confidential

import sha.config
from sha.contrib.monitor.abc_base import Monitor
from sha.config import ENABLED_MONITORS
from sha.constants import MONITER_METRIC_REPORTED, MONITER_METRIC_FAIL_REPORT

class TraceMonitor(Monitor):
    """
    TraceMonitor is a monitor for receiving opentracing spans.
    """
    def __init__(self, *args, **kwargs):
        self._logger = sha.config.get_logger(__name__)
        self._internal_monitor = None
        self._init_internal_monitor()
        self._metric_name = ENABLED_MONITORS['TraceMonitor']
        self._stats = {}

    def _init_internal_monitor(self):
        from sha.contrib.monitor.traceCollector.internal import IntTraceMonitor
        self._logger.info("start TraceMonitor")
        self._internal_monitor = IntTraceMonitor()

    def _stats_inc(self, field):
        self._stats[field] =  self._stats.get(field, 0) + 1

    def _notify_whiteboard(self, *args):
        key = "nsx.trace.spans"
        value = args[0]
        from sha.core.whiteboard import DataWhiteboard
        if not DataWhiteboard.report_data(key, value):
             self._stats_inc(MONITER_METRIC_REPORTED)
        else:
             self._stats_inc(MONITER_METRIC_FAIL_REPORT)

    def start(self):
        if not self._internal_monitor.notify:
            self._internal_monitor.notify = self._notify_whiteboard
        self._internal_monitor.start()

    def stop(self):
        self._logger.info("TraceMonitor stop.")
        self._internal_monitor.stop()

    def join(self):
        self._logger.info("TraceMonitor join.")
        self._internal_monitor.wait_to_stop()

    @property
    def internal_monitor(self):
        return self._internal_monitor

    @property
    def metric_name(self):
        return self._metric_name
