# Copyright (C) 2020 - 2021 VMware, Inc. All rights reserved.
# -- VMware Confidential

logger.debug("data_store:{}".format(data_store))

profile = data_store['profile']
sys_stats = data_store['metrics']['nsx.common.sys-stats']

def encap_stats(stats, msg):
    """System Stats"""
    try:
        mem_stats = stats.get('mem', None)
        kernel_stats = stats.get('kernel', None)
        processes_stats = stats.get('processes', None)
        system_stats = stats.get('system', None)
        swap_stats = stats.get('swap', None)
        if not mem_stats or not kernel_stats or not processes_stats or not system_stats or not swap_stats:
            logger.info("mem_stats {} kernel_stats {} processes_stats {} system_stats {} swap_stats {}"\
                        .format(mem_stats, kernel_stats, processes_stats, system_stats, swap_stats))
            return False
        tmp_msg = nsx_sys_metrics_stats_pb2.NsxSystemStats(obj_id = "system_stats",
                                                           mem_active = mem_stats['active'],
                                                           mem_available = mem_stats['available'],
                                                           mem_total = mem_stats['total'],
                                                           mem_buffered = mem_stats['buffered'],
                                                           mem_cached = mem_stats['cached'],
                                                           mem_dirty = mem_stats['dirty'],
                                                           mem_free = mem_stats['free'],
                                                           mem_huge_page_size = mem_stats['huge_page_size'],
                                                           mem_huge_pages_free = mem_stats['huge_pages_free'],
                                                           mem_huge_pages_total = mem_stats['huge_pages_total'],
                                                           mem_page_tables = mem_stats['page_tables'],
                                                           mem_swap_cached = mem_stats['swap_cached'],
                                                           mem_swap_free = mem_stats['swap_free'],
                                                           mem_swap_total = mem_stats['swap_total'],
                                                           boot_time = kernel_stats['boot_time'],
                                                           proc_running = processes_stats['running'],
                                                           proc_sleeping = processes_stats['sleeping'],
                                                           proc_blocked = processes_stats['blocked'],
                                                           proc_dead = processes_stats['dead'],
                                                           proc_idle = processes_stats['idle'],
                                                           proc_paging = processes_stats['paging'],
                                                           proc_stopped = processes_stats['stopped'],
                                                           proc_unknown = processes_stats['unknown'],
                                                           proc_zombies = processes_stats['zombies'],
                                                           proc_total = processes_stats['total'],
                                                           proc_total_threads = processes_stats['total_threads'],
                                                           load = system_stats['load1'],
                                                           n_cpus = system_stats['n_cpus'],
                                                           uptime = system_stats['uptime'],
                                                           swap_total = swap_stats['total'],
                                                           swap_free = swap_stats['free'])
        msg.system.CopyFrom(tmp_msg)
        """Procstat Stats"""
        procstat_stats = stats.get('procstat', None)
        if not procstat_stats:
            logger.info("procstat_stats {}".format(procstat_stats))
            return False
        for key, val in procstat_stats.items():
            tmp_msg = nsx_sys_metrics_stats_pb2.NsxProcstatStats(obj_id = key,
                                                                 cpu_usage = val['cpu_usage'],
                                                                 memory_usage = val['memory_usage'])
            sub_msg = msg.procstat.add()
            sub_msg.CopyFrom(tmp_msg)
        """Net Stats"""
        net_stats = stats.get('net', None)
        if not net_stats:
            logger.info("net_stats {}".format(net_stats))
            return False
        for key, val in net_stats.items():
            tmp_msg = nsx_sys_metrics_stats_pb2.NsxNetStats(obj_id = key,
                                                            bytes_sent = val['bytes_sent'],
                                                            bytes_recv = val['bytes_recv'],
                                                            packets_sent = val['packets_sent'],
                                                            packets_recv = val['packets_recv'],
                                                            errin = val['errin'],
                                                            errout = val['errout'],
                                                            dropin = val['dropin'],
                                                            dropout = val['dropout'])
            sub_msg = msg.net.add()
            sub_msg.CopyFrom(tmp_msg)
        """Disk Stats"""
        diskio_stats = stats.get('diskio', None)
        disk_stats = stats.get('disk', None)
        if not diskio_stats or not disk_stats:
            logger.info("diskio_stats {} disk_stats {}".format(diskio_stats, disk_stats))
            return False
        for key, val in diskio_stats.items():
            tmp_msg = nsx_sys_metrics_stats_pb2.NsxDiskStats(obj_id = key,
                                                             io_time = val['io_time'],
                                                             iops_in_progress = val['iops_in_progress'],
                                                             read_time = val['read_time'],
                                                             weighted_io_time = val['weighted_io_time'],
                                                             write_time = val['write_time'],
                                                             free = disk_stats[key]['free'],
                                                             total = disk_stats[key]['total'],
                                                             used = disk_stats[key]['used'])
            sub_msg = msg.disk.add()
            sub_msg.CopyFrom(tmp_msg)
        """Cpu Stats"""
        cpu_stats = stats.get('cpu', None)
        if not cpu_stats:
            logger.info("cpu_stats {}".format(cpu_stats))
            return False
        for key, val in cpu_stats.items():
            """
            tag = val.get('tag', None)
            obj_id = key + '/' + tag if tag else key
            """
            tmp_msg = nsx_sys_metrics_stats_pb2.NsxCpuStats(obj_id = key,
                                                            user = val['user'],
                                                            system = val['system'],
                                                            idle = val['idle'],
                                                            iowait = val['iowait'],
                                                            nice = val['nice'],
                                                            irq = val['irq'],
                                                            softirq = val['softirq'],
                                                            guest = val['guest'])
            sub_msg = msg.cpu.add()
            sub_msg.CopyFrom(tmp_msg)
        logger.debug("encapsulated message is {}".format(msg))
        return True
    except Exception as e:
        logger.info("system_stats: encapsulation failed as {} {}"\
                    .format(e, traceback.format_exc()))
        return False

def handle_stats(stats):
    entity_type = options_pb2.MetricsEntityType.Value('POLICY_EDGE_NODE') if stats[1] == "nsx-edge" else \
                  options_pb2.MetricsEntityType.Value('CLUSTER_NODE')
    msg = metrics_agent_rpc_pb2.MetricsAgentMessage(timestamp = int(sys_stats['timestamp']),
                                                    entity = entity_type,
                                                    entity_id = str(data_store['host_id']))
    if encap_stats(stats[0], msg):
        Exportdata(data = msg, exports = ['tsdb_metrics'])

if sys_stats and sys_stats['value']:
    logger.debug("sys_stats is {}".format(sys_stats['value']))
    handle_stats(sys_stats['value'])
else:
    logger.info("No sys-stats stats for exporting.")
